//----------------------------------------------------------------------------
// TSplitWindow v. 1.02 - (C) Copyright 1995 by Kent Reisdorph
//
// July 24, 1995 (modified 10/31/95)
//
// Source for class TSplitWindow, a class allow horizontal
// or vertical splitting and resizing of two windows.
//
//----------------------------------------------------------------------------
#include "splitwin.h"

DEFINE_RESPONSE_TABLE1(TSplitWindow, TLayoutWindow)
	EV_WM_LBUTTONDOWN,
	EV_WM_LBUTTONUP,
	EV_WM_MOUSEMOVE,
	EV_WM_SETCURSOR,
END_RESPONSE_TABLE;

// Constructor. Sets up child windows. Sets Layout.
TSplitWindow::TSplitWindow(TWindow* parent, const char far* title,
		TWindow* child1, TWindow* child2, int child1Size,
		bool vertical, bool isfixed, TModule* module)
	: TLayoutWindow(parent, title, module)
{
	childLeft = child1;
	TListBox* child = dynamic_cast<TListBox*>(childLeft);
	if (child) childLeft->Attr.Style |= LBS_NOINTEGRALHEIGHT;
	childLeft->SetParent(this);
	childLeft->EnableAutoCreate();
	childRight = child2;
	child = dynamic_cast<TListBox*>(childRight);
	if (child) childRight->Attr.Style |= LBS_NOINTEGRALHEIGHT;
	childRight->SetParent(this);
	childRight->EnableAutoCreate();
	leftChildSize = child1Size;
	isVertical = vertical;
	isFixedSize = isfixed;
	dragging = false;
	dc = 0;
	minWndSize = 5;
}

// copy constructor
TSplitWindow::TSplitWindow(const TSplitWindow& sw)
	: TLayoutWindow(0)
{
	isVertical = sw.isVertical;
	childLeft = sw.childLeft;
	childRight = sw.childRight;;
	oldPoint = sw.oldPoint;
	sizingPen = sw.sizingPen;
	dragging = sw.dragging;
	offScreen = sw.offScreen;
	leftChildSize = sw.leftChildSize;
	clientHeight = sw.clientHeight;
	clientWidth = sw.clientWidth;
	arrowType = sw.arrowType;
	dc = sw.dc;
	minWndSize = sw.minWndSize;
	isFixedSize = sw.isFixedSize;
}

void
TSplitWindow::SetupWindow()
{
	TLayoutWindow::SetupWindow();
	if (isVertical) LayoutVertical();
	else LayoutHorizontal();
}

// for direct containers
TSplitWindow&
TSplitWindow::operator = (const TSplitWindow& sw)
{
	isVertical = sw.isVertical;
	childLeft = sw.childLeft;
	childRight = sw.childRight;;
	oldPoint = sw.oldPoint;
	sizingPen = sw.sizingPen;
	dragging = sw.dragging;
	offScreen = sw.offScreen;
	leftChildSize = sw.leftChildSize;
	clientHeight = sw.clientHeight;
	clientWidth = sw.clientWidth;
	arrowType = sw.arrowType;
	dc = sw.dc;
	minWndSize = sw.minWndSize;
	isFixedSize = sw.isFixedSize;
	return *this;
}


// Operator == for use in container classes
bool
TSplitWindow::operator == (const TSplitWindow& sw) const
{
	return
	isVertical == sw.isVertical &&
	childLeft == sw.childLeft &&
	childRight == sw.childRight &&
	oldPoint == sw.oldPoint &&
	sizingPen == sw.sizingPen &&
	dragging == sw.dragging &&
	offScreen == sw.offScreen &&
	leftChildSize == sw.leftChildSize &&
	clientHeight == sw.clientHeight &&
	clientWidth == sw.clientWidth &&
	arrowType == sw.arrowType &&
	dc == sw.dc &&
	minWndSize == sw.minWndSize &&
  isFixedSize == sw.isFixedSize;
}

TSplitWindow::~TSplitWindow()
{
	delete dc;  // just in case
}

// EvLButtonDown starts the dragging process. Create a dc for
// our use while we are dragging the sizing bar. Sets ROP code
// to R2_NOT so that the window underneath the bar is visible
// through the bar. This mode also makes erasing the bar as
// simple as redrawing it at the same location.
void
TSplitWindow::EvLButtonDown(uint, TPoint& point)
{
	offScreen = false;
	SetCapture();
	dragging = true;
	dc = new TWindowDC(*this);
	dc->SetROP2(R2_NOT);
	TRect rect = GetClientRect();
	// Need client area width and height so we know how far to draw
	// the sizing bar.
	clientHeight = rect.Height();
	clientWidth = rect.Width();
	// Select a black pen the width of the sizing border.
	dc->SelectObject(TPen(TColor::Black, GetSystemMetrics(SM_CXFRAME)));
	// Draw vertical bar if isVertical == true otherwise draw a
	// horizontal bar.
	if (isVertical) {
		dc->MoveTo(point.x, 0);
		dc->LineTo(point.x, clientHeight);
	}
	else {
		dc->MoveTo(0, point.y);
		dc->LineTo(clientWidth, point.y);
	}
	// Need to save this TPoint so we can use it to erase the
	// line later.
	oldPoint = point;
}

// EvLButtonUp. Stops the dragging process. Resets the layout of
// the windows. Erases the last line drawn.
void
TSplitWindow::EvLButtonUp(uint, TPoint& point)
{
	dragging = false;
	if (dc) {
		if (isVertical) {
			dc->MoveTo(oldPoint.x, 0);
			dc->LineTo(oldPoint.x, clientHeight);
		}
		else {
			dc->MoveTo(0, oldPoint.y);
			dc->LineTo(clientWidth, oldPoint.y);
		}
	}
	ReleaseCapture();
	SetNewPositions(point);
	delete dc;
	dc = NULL;
}

// EvMouseMove. Erases and redraws the line. Sets the cursor
// back to the arrow if it moves off our window. Resets the
// cursor to the sizing cursor when the cursor moves back over
// our window. Calls DrawSizingLine for actual line drawing.
void
TSplitWindow::EvMouseMove(uint, TPoint& point)
{
	if (!dragging) return;
	if (point.x < 0 || point.x > clientWidth ||
			point.y < 0 || point.y > clientHeight) {
		if (!offScreen) ::SetCursor(LoadCursor(NULL, IDC_ARROW));
		offScreen = true;
	}
	else if (offScreen) {
		offScreen = false;
		::SetCursor(LoadCursor(NULL, arrowType));
	}
	if (!offScreen) DrawSizingBar(point);
}

// Sets the cursor to the sizing cursor if it is over our
// window as opposed to one of the child windows.
bool
TSplitWindow::EvSetCursor(HWND hwnd, uint, uint msg)
{
	if (msg == WM_MOUSEMOVE)
		if (hwnd == *this) ::SetCursor(LoadCursor(NULL, arrowType));
	return false;
}

// Layout for Vertical split window. Default setup is 50% for
// each window. The gap is determined by the size of a sizeable
// frame - (2 * the size of a non-sizeable border). The size
// SM_CXFRAME was too large. The top left corner of the left
// child is anchored at 0,0. The bottom is the bottom of the parent.
// The width is set to 50% of the parent. The right child is
// the width of the gap to the right of the left child. The top
// and bottom edges are set to the size of the parent. The right
// edge is set to the right edge of the parent.
// SetChildLayoutMetrics sets the metrics for the child and
// TLayoutWindow::Layout() sets the layout once the layout metrics
// are set for each child.
void
TSplitWindow::LayoutVertical()
{
	TLayoutMetrics lm;
	childLeft->Attr.Style |= WS_CLIPSIBLINGS | WS_BORDER;
	lm.X.Absolute(lmLeft, 0);
	lm.Y.Absolute(lmTop, 0);
	if (isFixedSize) {
		lm.SetMeasurementUnits(lmPixels);
		long w = Parent->GetClientRect().Width();
		lm.Width.Absolute(lmRight, (int)(((long)leftChildSize * w)/100));
	}
	else lm.Width.PercentOf(lmParent, leftChildSize);
	lm.Height.SameAs(lmParent, lmHeight);
	SetChildLayoutMetrics(*childLeft, lm);
	childRight->Attr.Style |= WS_CLIPSIBLINGS | WS_BORDER;
	int gap = GetSystemMetrics(SM_CXFRAME) -
		(GetSystemMetrics(SM_CYBORDER) *2);
	lm.X.RightOf(childLeft, gap);
	lm.Y.Absolute(lmTop, 0);
	lm.Width.SameAs(lmParent, lmRight);
	lm.Height.SameAs(lmParent, lmHeight);
	SetChildLayoutMetrics(*childRight, lm);
	Layout();
	arrowType = (char*)IDC_SIZEWE;
}

// Same as above only different.
void
TSplitWindow::LayoutHorizontal()
{
	TLayoutMetrics lm;
	childLeft->Attr.Style |= WS_CLIPSIBLINGS | WS_BORDER;
	lm.X.Absolute(lmLeft, 0);
	lm.Y.Absolute(lmTop, 0);
	lm.Width.SameAs(lmParent, lmWidth);
	if (isFixedSize) {
		lm.SetMeasurementUnits(lmPixels);
		long h = Parent->GetClientRect().Height();
		lm.Height.Absolute(lmBottom, (int)(((long)leftChildSize * h)/100));
	}
	else lm.Height.PercentOf(lmParent, leftChildSize);
	SetChildLayoutMetrics(*childLeft, lm);
	childRight->Attr.Style |= WS_CLIPSIBLINGS | WS_BORDER;
	int gap = GetSystemMetrics(SM_CXFRAME) -
		(GetSystemMetrics(SM_CYBORDER) *2);
	lm.X.Absolute(lmLeft, 0);
	lm.Y.Below(childLeft, gap);
	lm.Width.SameAs(lmParent);
	lm.Height.SameAs(lmParent, lmBottom);
	SetChildLayoutMetrics(*childRight, lm);
	Layout();
	arrowType = (char*)IDC_SIZENS;
}

// Switches from a vertical split window to a horizontal split
// window and vice versa.
void
TSplitWindow::Switch()
{
	if (isVertical) {
		isVertical = false;
		LayoutHorizontal();
	}
	else {
		isVertical = true;
		LayoutVertical();
	}
}

// SetNewPositions resets the layout metric for each child and
// calls Layout() to realize the changes. The layout is set as
// a percentage of the parent. Future implementations might want
// the layout to be an absolute location rather than a percentage.
void
TSplitWindow::SetNewPositions(TPoint& point)
{
	TRect rect = GetWindowRect();
	TLayoutMetrics lm;
	lm.X.Absolute(lmLeft, 0);
	lm.Y.Absolute(lmTop, 0);
	if (isVertical) {
		if (point.x < minWndSize) point.x = minWndSize;
		if (point.x > rect.Width() - minWndSize)
			point.x = rect.Width() - minWndSize;
		leftChildSize = (int)((point.x*100L)/rect.Width());
		if (isFixedSize) {
			long w = Parent->GetClientRect().Width();
			lm.SetMeasurementUnits(lmPixels);
			lm.Width.Absolute(lmRight, (int)(((long)leftChildSize * w)/100));
		}
		else lm.Width.PercentOf(lmParent, leftChildSize);
		lm.Height.SameAs(lmParent, lmHeight);
	}
	else {
		if (point.y < minWndSize) point.y = minWndSize;
		if (point.y > rect.Height() - minWndSize)
			point.y = rect.Height() - minWndSize;
		leftChildSize = (int)((point.y*100L)/rect.Height());
		lm.Width.SameAs(lmParent, lmWidth);
		if (isFixedSize) {
			lm.SetMeasurementUnits(lmPixels);
			long h = Parent->GetClientRect().Height();
			lm.Height.Absolute(lmBottom, (int)(((long)leftChildSize * h)/100));
		}
		else lm.Height.PercentOf(lmParent, leftChildSize);
	}
	SetChildLayoutMetrics(*childLeft, lm);
	Layout();
}

// DrawSizingBar. First erases the old line by drawing a line at
// the old location. Then we draw a line at the new location and
// save the new location in oldPoint so we can erase later.
void
TSplitWindow::DrawSizingBar(TPoint& point)
{
	TRect rect = GetWindowRect();
	if (isVertical) {
		if (point.x < minWndSize) point.x = minWndSize;
		if (point.x > rect.Width() - minWndSize)
			point.x = rect.Width() - minWndSize;
		dc->MoveTo(oldPoint.x, 0);
		dc->LineTo(oldPoint.x, clientHeight);
		dc->MoveTo(point.x, 0);
		dc->LineTo(point.x, clientHeight);
	}
	else {
		if (point.y < minWndSize) point.y = minWndSize;
		if (point.y > rect.Height() - minWndSize)
			point.y = rect.Height() - minWndSize;
		dc->MoveTo(0, oldPoint.y);
		dc->LineTo(clientWidth, oldPoint.y);
		dc->MoveTo(0, point.y);
		dc->LineTo(clientWidth, point.y);
	}
	oldPoint = point;
}

// SetLeftChild. Deletes the old client and creates a new one
// returns a pointer to the new child. Returns 0 if we were passed
// a 0 as the new window. Without this the app will crash on the
// SetParent() call. Layout the windows as we do in the constructor.
TWindow*
TSplitWindow::SetLeftChild(TWindow* client)
{
	if (!client) return 0;
	if (childLeft->CanClose() == false) return 0;
	delete childLeft;
	childLeft = NULL;
	childLeft = client;
	childLeft->SetParent(this);
	childLeft->EnableAutoCreate();
	childLeft->Create();
	if (childLeft) {
		if (isVertical) LayoutVertical();
		else LayoutHorizontal();
	}
	return childLeft;
}

// SetRightClient. Same as SetLeftChild above.
TWindow*
TSplitWindow::SetRightChild(TWindow* client)
{
	if (!client) return 0;
	if (childRight->CanClose() == false) return 0;
	delete childRight;
	childRight = NULL;
	childRight = client;
	childRight->SetParent(this);
	childRight->EnableAutoCreate();
	childRight->Create();
	if (childRight) {
		if (isVertical) LayoutVertical();
		else LayoutHorizontal();
	}
	return childRight;
}

// NextChild. Sets the focus to a specified child window.
// Called without a child parameter this function will switch
// focus to the next child window. Called with a LEFT_CHILD
// or RIGHT_CHILD constant it will set focus to that child.
void
TSplitWindow::NextChild(int child)
{
	if (!child) {
		if (GetFocus() == *childLeft) ::SetFocus(*childRight);
		else ::SetFocus(*childLeft);
	}
	else {
		if (child == LEFT_CHILD) ::SetFocus(*childLeft);
		else ::SetFocus(*childRight);
	}
}


